(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs-interop')) :
    typeof define === 'function' && define.amd ? define(['exports', 'rxjs-interop'], factory) :
    (global = global || self, factory(global.subscribableThings = {}, global.rxjsInterop));
}(this, (function (exports, rxjsInterop) { 'use strict';

    var createMediaQueryMatch = function createMediaQueryMatch(emitNotSupportedError, window, wrapSubscribeFunction) {
      return function (mediaQueryString) {
        return wrapSubscribeFunction(function (observer) {
          if (window === null || window.matchMedia === undefined) {
            return emitNotSupportedError(observer);
          }

          var mediaQueryList = window.matchMedia(mediaQueryString);
          observer.next(mediaQueryList.matches);

          mediaQueryList.onchange = function () {
            return observer.next(mediaQueryList.matches);
          };

          return function () {
            mediaQueryList.onchange = null;
          };
        });
      };
    };

    var createMutations = function createMutations(emitNotSupportedError, window, wrapSubscribeFunction) {
      return function (htmlElement, options) {
        return wrapSubscribeFunction(function (observer) {
          if (window === null || window.MutationObserver === undefined) {
            return emitNotSupportedError(observer);
          }

          var mutationObserver = new window.MutationObserver(function (records) {
            return observer.next(records);
          });

          try {
            mutationObserver.observe(htmlElement, options);
          } catch (err) {
            observer.error(err);
          }

          return function () {
            return mutationObserver.disconnect();
          };
        });
      };
    };

    var createPermissionState = function createPermissionState(emitNotSupportedError, window, wrapSubscribeFunction) {
      return function (permissionDescriptor) {
        return wrapSubscribeFunction(function (observer) {
          if (window === null || window.navigator === undefined || window.navigator.permissions === undefined || window.navigator.permissions.query === undefined) {
            return emitNotSupportedError(observer);
          }

          var isActive = true;

          var unsubscribe = function unsubscribe() {
            isActive = false;
          };

          window.navigator.permissions.query(permissionDescriptor).then(function (permissionStatus) {
            if (isActive) {
              observer.next(permissionStatus.state);

              permissionStatus.onchange = function () {
                return observer.next(permissionStatus.state);
              };

              unsubscribe = function unsubscribe() {
                permissionStatus.onchange = null;
              };
            }
          })["catch"](function (err) {
            if (isActive) {
              observer.error(err);
            }
          });
          return function () {
            return unsubscribe();
          };
        });
      };
    };

    var createReports = function createReports(emitNotSupportedError, window, wrapSubscribeFunction) {
      return function (options) {
        return wrapSubscribeFunction(function (observer) {
          if (window === null || window.ReportingObserver === undefined) {
            return emitNotSupportedError(observer);
          }

          var reportingObserver = new window.ReportingObserver(function (reportList) {
            return observer.next(reportList);
          }, options);
          reportingObserver.observe();
          return function () {
            return reportingObserver.disconnect();
          };
        });
      };
    };

    // @todo TypeScript does not include type definitions for the Reporting API yet.
    var createWindow = function createWindow() {
      return typeof window === 'undefined' ? null : window;
    };

    var createWrapSubscribeFunction = function createWrapSubscribeFunction(patch, toObserver) {
      var emptyFunction = function emptyFunction() {}; // tslint:disable-line:no-empty


      return function (innerSubscribe) {
        var _subscribe = function subscribe() {
          var unsubscribe = innerSubscribe(toObserver.apply(void 0, arguments));

          if (unsubscribe !== undefined) {
            return unsubscribe;
          }

          return emptyFunction;
        };

        _subscribe[Symbol.observable] = function () {
          return {
            subscribe: function subscribe() {
              return {
                unsubscribe: _subscribe.apply(void 0, arguments)
              };
            }
          };
        };

        return patch(_subscribe);
      };
    };

    var emitNotSupportedError = function emitNotSupportedError(observer) {
      observer.error(new Error('The required browser API seems to be not supported.'));
      return function () {}; // tslint:disable-line:no-empty
    };

    var window$1 = createWindow();
    var wrapSubscribeFunction = createWrapSubscribeFunction(rxjsInterop.patch, rxjsInterop.toObserver);
    var mediaQueryMatch = createMediaQueryMatch(emitNotSupportedError, window$1, wrapSubscribeFunction);
    var mutations = createMutations(emitNotSupportedError, window$1, wrapSubscribeFunction);
    var permissionState = createPermissionState(emitNotSupportedError, window$1, wrapSubscribeFunction);
    var reports = createReports(emitNotSupportedError, window$1, wrapSubscribeFunction);

    exports.mediaQueryMatch = mediaQueryMatch;
    exports.mutations = mutations;
    exports.permissionState = permissionState;
    exports.reports = reports;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
